
%%%   WKM algorithm   %%%%%%%
%%% The algorithm from following paper: J. Z. Huang, M. K. Ng, H. Rong, and
%%% Z. Li, "Automated variable weighting in k-means type clustering", IEEE
%%% Transactions on Pattern Analysis and Machine Intelligence, vol. 27, no.
%%% 5, pp. 657-668, May 2005.

%%%% This experiment investigate the performance of W-K-Modes algorithm on
%%%% categorical data.

function [ACC,ARI,NMI] = KM_main(x,Modes)

%%
[n,d]=size(x);
true_class=(x(:,d));
true_class = true_class';
d=d-1;
x=x(:,1:d);
k=max(true_class);
m=1;
for i=1:d
    t=max(x(:,i));
    if t>m
        m=t;
    end
end
Pm.Xlth=n;
Pm.Xwd=d;
Pm.k=k;
Pm.O=4; %*%*%*%*%*%*%*%* set this for different data sets
%%% 0 for pure nominal data, d for pure ordinal data %%%

%%
[x,Pm,AttMtx] = EBDMPre(x,Pm);

miu = Modes;  %%% k modes

%%
%%%%% Assign the data in the initial k clusters %%%%%%%%%%
cluster_label = zeros(1, n);
cluster_count = zeros(1, k);
%frequency_in_cluster = zeros(k, d, m);
for i = 1:n
    [DistVec] = EBDMDist_KM(x(i,:),miu,AttMtx,Pm);
    [~,j]=min(DistVec);
    cluster_label(i)=j;
    cluster_count(j) = cluster_count(j) + 1;
end
%%
%%%%%%% update the initial k modes %%%%%%%%%%%%%%%%%%%%%%%%%%%
[new_miu, ~] = Update_modes(cluster_label, x, k, m);

%%
%%%%%%%% Iterative steps: adjust the cluster label of each data and update the modes of corresponding clusters
%%%%%%%% untill no data need to change class. %%%%%%%%%%%%%%%%%%%%%%%%%%
moves = 1;
epoch = 0;
while (moves ~= 0)&&(epoch < 50) %%% some class label has been changed in the last step%%%
    moves = 0;
    %%
    %%%update the cluster label of all data
    for i = 1:n
        [DistVec] = EBDMDist_KM(x(i,:),new_miu,AttMtx,Pm);
        [~,new_label]=min(DistVec);
        if cluster_label(i) ~= new_label  %%% need adjust
            moves = moves + 1;
            old_cluster = cluster_label(i);
            cluster_label(i) = new_label;     %%%%updata cluster label
            cluster_count(new_label) = cluster_count(new_label) + 1;
            cluster_count(old_cluster) = cluster_count(old_cluster) - 1;   %%%%update cluster counter
        end
    end
    [new_miu, ~] = Update_modes(cluster_label, x, k, m);
    epoch = epoch + 1;
end

%%
%%%% Clustering accuracy
[ACC,Confu_Matrix,~] = calAccuracy(cluster_label',true_class');

%%
%%% ARI index
ARI = calRI(cluster_label,true_class,'adjusted');

%%
%%% NMI index
[NMI,~, ~,~,~] = calNMI(Confu_Matrix, n);

end
